/**
 * \file
 *
 * \brief irmp, irsnd and irthru test
 *
 */

/**
 * \mainpage irmp, irsnd and irthru test doxygen documentation
 *
 * \par irmp, irsnd and irthru test
 *
 * This program test the ir lib from Frank Meyer - frank(at)fli4l.de
 *
 * \par Content
 *
 * -# test irmp and put result on stdout (through asf.h)
 * -# redirect irmp to irsnd using USE_IRSND
 * -# redirect tsop level to ir send diode with default carrier frequency using USE_IRTHRU
 * -# enable IRMP_LOGGING = 1 to see the tsop states
 */

///< definieren, wenn durchreichen ohne Protokoll erkennung gewuenscht wird. 6586 bytes weniger code als irmp mit irsnd.
//#define USE_IRTHRU

#ifdef USE_IRTHRU
#  include <irthru.h>
#else
///< definieren, wenn durchreichen via irsnd gewuenscht wird
#  define USE_IRSND
#  include <irmp.h>
#endif

#ifdef ATMEL_ASF                     // ATMEL ASF
#define RC_TIMER       &TCC0
#define RC_TIMER_RES   4000000UL
#else
#error Platform not supported, add definititions for your platform
#endif

// dirty, don't include *.c files in real applications
#if defined(USE_IRSND)
#  include <irsnd.c>
#elif defined(USE_IRTHRU)
#  include <irthru.c>
#endif


#ifndef USE_IRTHRU
IRMP_DATA irmp_data;
#endif

static void rc_isr_handler(void)
{
  static uint16_t isr_count = 0;
#if defined(USE_IRTHRU)
  irthru_ISR();
#elif defined(USE_IRSND)
  if (! irsnd_ISR()) {
#if defined(BOARD) && BOARD == XMEGA_A1_XPLAINED
    // if not sending
    LED_Off(LED2);
#endif
    irmp_ISR();
  }
#else
  irmp_ISR();
#endif
  if (isr_count++ > F_INTERRUPTS) {
    isr_count = 0;
#if defined(BOARD) && BOARD == XMEGA_A1_XPLAINED
    // blinke im sekunden takt
    LED_Toggle(LED3);
#endif
  }
}

static void rc_init(void)
{
#ifdef USE_IRTHRU
  irthru_init();
#else
#  ifdef USE_IRSND
  irsnd_init();
#  endif
  irmp_init();
#endif
#ifdef ATMEL_ASF                     // ATMEL ASF
  irq_initialize_vectors(); // done by pmic_init()
  cpu_irq_enable();

  tc_enable(RC_TIMER);
  // Enable overflow interrupt
  tc_set_overflow_interrupt_level(RC_TIMER, TC_INT_LVL_LO);
  // Configure TC in normal mode
  tc_set_wgm(RC_TIMER, TC_WG_NORMAL);
  // Configure call back interrupt
  tc_set_overflow_interrupt_callback(RC_TIMER, rc_isr_handler);
  // Configure TC period and and resolution
  tc_set_resolution(RC_TIMER, RC_TIMER_RES);
  uint32_t reso = tc_get_resolution(RC_TIMER);
  if (reso != RC_TIMER_RES) {
    printf("warning timer resolution not same actual=%ld chosen=%ld period=%ld",
      reso, RC_TIMER_RES, (reso / F_INTERRUPTS));
  }
  tc_write_period(RC_TIMER, (reso / F_INTERRUPTS));
#else
#error Platform not supported, add timer initialisation for your platform
#endif
}

#ifndef USE_IRTHRU
static void show_rc_data(void)
{
  static uint16_t rc_repeat_count = 0;
  if (irmp_data.flags & IRMP_FLAG_REPETITION) {
    // user immer noch auf gleicher taste
#if defined(BOARD) && BOARD == XMEGA_A1_XPLAINED
      LED_Toggle(LED0);
#endif
    if (rc_repeat_count++ > 3) {
//      printf("R");
#if defined(BOARD) && BOARD == XMEGA_A1_XPLAINED
      LED_Toggle(LED1);
#endif
    }
  }
  else {
    // erster tastendruck
#if defined(BOARD) && BOARD == XMEGA_A1_XPLAINED
    LED_On(LED0);
    LED_Off(LED1);
#endif
    rc_repeat_count = 0;
#ifdef USE_IRSND
    irmp_data.flags = 0;    // reset flags!
#if defined(BOARD) && BOARD == XMEGA_A1_XPLAINED
    LED_On(LED2);
#endif
    irsnd_send_data(&irmp_data, 0);
#endif
    printf("\n%d", irmp_data.protocol);
#if (IRMP_PROTOCOL_NAMES  == 1)       // 1: access protocol names, 0: do not (default),
    printf(" %s", irmp_protocol_names[irmp_data.protocol]);
#endif
    uint16_t offset = 0;
    if (irmp_data.protocol == IRMP_RC6A_PROTOCOL) {
      offset = 1024;
    }
    printf(" adr=%d cmd=%d ", irmp_data.address, (irmp_data.command - offset));
#if IRMP_LOGGING == 1
    printf(" freq=%ld\n", F_INTERRUPTS);
#endif
  }
}
#endif

int main(void)
{
#ifdef ATMEL_ASF                     // ATMEL ASF
  const usart_serial_options_t usart_serial_options = {
    .baudrate     = USART_SERIAL_BAUDRATE,
    .charlength   = USART_SERIAL_CHAR_LENGTH,
    .paritytype   = USART_SERIAL_PARITY,
    .stopbits     = USART_SERIAL_STOP_BIT
  };

  pmic_init();
  board_init();
  sysclk_init();
  stdio_serial_init(USART_SERIAL, &usart_serial_options);
  sleepmgr_init();
#endif
  rc_init();

#ifdef USE_IRTHRU
  do {
#else
  do {
    if (irmp_get_data(&irmp_data)) {
      show_rc_data();
    }
#endif
#ifdef ATMEL_ASF                     // ATMEL ASF
    // Sleep until interrupt triggers.
    sleepmgr_enter_sleep();
#endif
  } while (1);
}
